/*
 * Generate a VGA signal using a PIC32 microcontroller.
 *
 * Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "pic32_c.h"
#include "init.h"
#include "debug.h"

/* Specific functionality. */

#include "main.h"
#include "devconfig.h"
#include "vga.h"
#include "display.h"
#include "display_config.h"
#include "vga_display.h"



/* Blink an attached LED with delays implemented using a loop. */

static void blink(uint32_t delay, uint32_t port, uint32_t pins)
{
    uint32_t counter;

    /* Clear outputs (LED). */

    CLR_REG(port, pins);

    while (1)
    {
        counter = delay;

        while (counter--) __asm__("");	/* retain loop */

        /* Invert outputs (LED). */

        INV_REG(port, pins);
    }
}



/* Main program. */

void main(void)
{
    init_memory();
    init_pins();
    init_outputs();

    unlock_config();
    config_oc();
    config_uart();
    lock_config();

    init_dma();

    /* Initialise VGA output structures with two line channels and no initiating
       channel. */

    init_vga(&display_config, 2, -1);

    /* Configure VGA output transfer details along with a timer and output
       compare units for horizontal and vertical sync. */

    vga_configure_transfer(T2, PORTB);
    vga_configure_sync(1, 2, 2);

    uart_init(1, FPB, 115200);
    uart_on(1);

    test_linedata(&display_config);

    interrupts_on();

    blink(3 << 24, PORTA, 1 << 3);
}



/* Exception and interrupt handlers. */

void exception_handler(void)
{
    blink(3 << 12, PORTA, 1 << 3);
}

void interrupt_handler(void)
{
    uint32_t ifs;

    /* Check for a OC1 interrupt condition. */

    ifs = REG(OCIFS) & OC_INT_FLAGS(1, OCxIF);

    if (ifs)
    {
        vga_interrupt_handler();
        CLR_REG(OCIFS, ifs);
    }
}



/* Peripheral pin configuration. */

void config_oc(void)
{
    /* Map OC1 to RPA0. */

    REG(RPA0R) = 0b0101;        /* RPA0R<3:0> = 0101 (OC1) */

    /* Map OC2 to RPA1. */

    REG(RPA1R) = 0b0101;        /* RPA1R<3:0> = 0101 (OC2) */
}

void config_uart(void)
{
    /* Map U1RX to RPB13. */

    REG(U1RXR) = 0b0011;        /* U1RXR<3:0> = 0011 (RPB13) */

    /* Map U1TX to RPB15. */

    REG(RPB15R) = 0b0001;       /* RPB15R<3:0> = 0001 (U1TX) */

    /* Set RPB13 to input. */

    SET_REG(TRISB, 1 << 13);
}
